==> dasu_2.02.jar: edu/northwestern/dasu/measurement/NetstatBWStats.java <==
/**
 * Dasu Project
 *
 * File:         NetstatBWStats.java
 * RCS:          $Id: NetstatBWStats.java,v 1.2 2010/11/01 15:37:00 mas939 Exp $
 * Description:  NetstatBWStats class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Sep 30, 2010 at 12:15:52 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.measurement
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package edu.northwestern.dasu.measurement;


import com.azureus.plugins.aznetmon.main.DataListener;

import edu.northwestern.dasu.util.Pair;
import edu.northwestern.dasu.util.Util;

import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.ArrayList;

import org.apache.commons.math.stat.univariate.rank.Percentile;


/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The NetstatBWStats class ...
 */


public class NetstatBWStats {


    public static long INTERVAL = 1000 * 30 * 1; //show a stat every 30 seconds.
    public static int MAX_NUM_ENTRIES = 100;

    //Some constants for other packages to use.
    public static final String PLUGIN_MAP_NAME = "plugin.versionserver.data";

    
    private static NetstatBWStats instance = new NetstatBWStats(INTERVAL);

    //data
    private List<NetstatBWData> bwMeasurementDataList;
    private List dataListeners; //<DataListener>
    private NetstatBWData baseBWData = null;

    private String status;


    public synchronized static NetstatBWStats getInstance() {
        return instance;
    }

    private NetstatBWStats(long interval)
    {
        bwMeasurementDataList = new ArrayList<NetstatBWData>(); //<BWData>
        dataListeners = new ArrayList<DataListener>(); //<DataListener>
        status = "Initial data is zero. Collection is 10 min interval.";
    }

    /**
     * What is the status of the application.
     * @param _status -
     */
    public void setStatus(String _status){
        status = _status;
    }

    public String getStatus(){
        return status;
    }

    
    public void add(NetstatBWData d){
    	
    	bwMeasurementDataList.add(0,d);
    	
    	if (baseBWData == null)
    		baseBWData = new NetstatBWData(d.timeRecorded, d.bytesSent, d.bytesReceived);
    	

        if (bwMeasurementDataList.size()>MAX_NUM_ENTRIES) 
        	bwMeasurementDataList.remove(bwMeasurementDataList.size()-1);
        
        //inform any listeners of this new data.
        update();

        //update the status
        setStatus( "Last measurement at "+ Util.currentGMTTime() + ". Next update in " + INTERVAL/1000 + " minutes" );

    }//add


    public NetstatBWData getMostRecent(){

        if( bwMeasurementDataList.size()>0 ){
            return (NetstatBWData) bwMeasurementDataList.get(0);
        }else{
            return null;
        }
    }

    public NetstatBWData getBaseBWData() {
    	return baseBWData;
    }
    
    
    public NetstatBWData getValAroundTime(long time) {

    	NetstatBWData bwData = null;
    	if( bwMeasurementDataList.size()>0 ){
    		for (int i = bwMeasurementDataList.size()-1; i >= 0; i--) {
    			if (Math.abs(( (NetstatBWData) bwMeasurementDataList.get(i)).timeRecorded - time )/1000 <= 5) {
    				bwData = (NetstatBWData) bwMeasurementDataList.get(i);
    				break;
    			}
    				
    			//return (NetstatBWData) bwMeasurementDataList.get(0);
    		}
    	}
    	
    	return bwData;
    	
    }
    

    /**
     * Get upto n most recent results.
     * @param n -
     * @return -
     */
    public NetstatBWData[] getMostRecent(int n){

        int size = bwMeasurementDataList.size();
        if( n<size ) size = n;

        if(size==0){
        	NetstatBWData[] def = new NetstatBWData[1];
            def[0] = new NetstatBWData();
            return def;
        }

        NetstatBWData[] retVal = new NetstatBWData[size];
        for(int i=0; i<size; i++){

            retVal[i] = (NetstatBWData) bwMeasurementDataList.get(i);

        }//for
        
        return retVal;
    }

    private ArrayList<NetstatBWData> getDataBetween(long first, long last) {
    	
    	ArrayList<NetstatBWData> retVal = new ArrayList<NetstatBWData>();
    	
    	Iterator<NetstatBWData> iter = bwMeasurementDataList.iterator();
    	while (iter.hasNext()) {
    		NetstatBWData data = iter.next();
    		if ((data.timeRecorded >= first) && (data.timeRecorded <= last))
    			retVal.add(data);
    	}

    	return retVal;
    }
    
    /* returns values in BytesPerSecond */
    public Double getPercentileUploadBetween(long first, long last, float percentile) {
		
    	ArrayList<NetstatBWData> vals = getDataBetween(first, last);
    	
    	double[] v = new double[vals.size()];
    	Double currSent = 0d;
    	Long currTimeSpan = 1l;
    	
    	for (int i=0; i < vals.size()-1; i++) {
			currSent = (double) (vals.get(i).bytesSent - vals.get(i+1).bytesSent);
			currTimeSpan = (vals.get(i).timeRecorded - vals.get(i+1).timeRecorded);
			v[i] = currSent/currTimeSpan;
    	}
    	
    	Percentile pt = new Percentile();
    	pt.setPercentile(percentile);
    	return pt.evaluate(v)*1000;
    	
    }
    
    /* returns values in BytesPerSecond */
    public Double getPercentileDownloadBetween(long first, long last, float percentile) {
		
    	ArrayList<NetstatBWData> vals = getDataBetween(first, last);
    	
    	double[] v = new double[vals.size()];
    	Double currReceived = 0d;
    	Long currTimeSpan = 1l;
    	
    	for (int i=0; i < vals.size()-1; i++) {
			currReceived = (double) (vals.get(i).bytesReceived - vals.get(i+1).bytesReceived);
			currTimeSpan = (vals.get(i).timeRecorded - vals.get(i+1).timeRecorded);
			v[i] = currReceived/currTimeSpan;
    	}
    	
    	Percentile pt = new Percentile();
    	pt.setPercentile(percentile);
    	return pt.evaluate(v)*1000;
    	
    }
    
    /* returns values in BytesPerSecond */
    public Double getFastestUploadBetween(long first, long last) {
    	
    	ArrayList<NetstatBWData> vals = getDataBetween(first, last);
    	Long timeSpan = 1l;
    	Double currSent = 0d;
    	Long currTimeSpan = 1l;
    	Double ret = new Double(-1);
    	
    	for (int i=0; i < vals.size()-1; i++) {

			currSent = (double) (vals.get(i).bytesSent - vals.get(i+1).bytesSent);
			currTimeSpan = (vals.get(i).timeRecorded - vals.get(i+1).timeRecorded);

			if (currTimeSpan != 0 && timeSpan != 0) {
				if (currSent/currTimeSpan > ret)
					ret = currSent/currTimeSpan;
			}
    	}
    	
    	return ret*1000;
    }
    
    /* returns values in BytesPerSecond */
  
    public Double getFastestDownloadBetween(long first, long last) {
    	    	
    	ArrayList<NetstatBWData> vals = getDataBetween(first, last);
    	Long timeSpan = 1l;
    	Double currReceived = 0d;
    	Long currTimeSpan = 1l;
    	Double ret = new Double(-1);
    	
    	for (int i=0; i < vals.size()-1; i++) {

    		currReceived = (double) (vals.get(i).bytesReceived - vals.get(i+1).bytesReceived);
			currTimeSpan = (vals.get(i).timeRecorded - vals.get(i+1).timeRecorded);
			
			if (currTimeSpan != 0 && timeSpan != 0) {
				if (currReceived/currTimeSpan > ret)
					ret = currReceived/currTimeSpan;
			}
    	}
    	
    	return ret*1000;
    }
    
    public synchronized void update(){

        int size = dataListeners.size();
        for(int i=0; i<size; i++ ){
            DataListener dl = (DataListener) dataListeners.get(i);
            dl.update();
        }

    }//update

    @Override
    public String toString() {
    	
    	String ret = new String();
    	Iterator<NetstatBWData> iter = bwMeasurementDataList.iterator();
    	while (iter.hasNext()) {
    		NetstatBWData data = iter.next();
    		ret = ret.concat(data.toString());
    	}
    	
    	return ret;
    }
    
    
    /**
     * data-type.
     */
    public static class NetstatBWData{

    	public long timeRecorded=0;
        public String msg = "";

        public long bytesSent=-1;
        public long bytesReceived=-1;
        
        public long deltaTime=0;
        public long deltaBytesSent=0;
        public long deltaBytesReceived=0;

        //Note: This is used for debugging only.
        public boolean isDebugMode = false;
        public String[] debugLines = null;
        
        public NetstatBWData() {
        	timeRecorded = Util.currentGMTTime();
        }
        
        public NetstatBWData(long sent, long received){
        	timeRecorded = Util.currentGMTTime();
        	bytesSent = sent;
        	bytesReceived = received;
        }
        
        public NetstatBWData(long timeRecorded, long sent, long received){
        	this.timeRecorded = timeRecorded;
        	this.bytesSent = sent;
        	this.bytesReceived = received;
        }
 

        public NetstatBWData(String _msg){
            msg = _msg;
        }

        /**
         * Calculate percentages and deltas based on the raw data.
         * @param prev -
         */
        public void calculate(NetstatBWData prev){

            deltaTime = timeRecorded - prev.timeRecorded;
            deltaBytesSent = bytesSent - prev.bytesSent;
            deltaBytesReceived = bytesReceived - prev.bytesReceived;

        }
        
        @Override
        public String toString() {
        	
        	return "timeRecorded: " + timeRecorded + " bytesSent: " + bytesSent +
        		" bytesReceived: " + bytesReceived + "\n";
        }
        
    }//static class NetstatBWData


}//class

==> dasu_3.02.jar: edu/northwestern/dasu/measurement/NetstatBWStats.java <==
/**
 * Dasu Project
 *
 * File:         NetstatBWStats.java
 * RCS:          $Id$
 * Description:  NetstatBWStats class (see below)
 * Author:       Mario Sanchez
 *               Northwestern Systems Research Group
 *               Department of Computer Science
 *               Northwestern University
 * Created:      Sep 30, 2010 at 12:15:52 PM
 * Language:     Java
 * Package:      edu.northwestern.dasu.measurement
 * Status:       Experimental (Do Not Distribute)
 *
 * (C) Copyright 2010, Northwestern University, all rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

package edu.northwestern.dasu.measurement;

import edu.northwestern.dasu.util.Util;

import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;

import org.apache.commons.math.stat.univariate.rank.Percentile;


/**
 * @author Mario Sanchez &lt;msanchez@eecs.northwestern.edu&gt;
 *
 * The NetstatBWStats class ...
 */


public class NetstatBWStats {

    public static int MAX_NUM_ENTRIES = 100;
    
    private static NetstatBWStats instance = new NetstatBWStats();

    //data
    private List<NetstatBWData> bwMeasurementDataList;
    private NetstatBWData baseBWData = null;

    public synchronized static NetstatBWStats getInstance() {
        return instance;
    }

    private NetstatBWStats()
    {
        bwMeasurementDataList = new ArrayList<NetstatBWData>(); //<BWData>
    }
    
    public void add(NetstatBWData d){
    	
    	bwMeasurementDataList.add(0,d);
    	
    	if (baseBWData == null)
    		baseBWData = new NetstatBWData(d.timeRecorded, d.bytesSent, d.bytesReceived);
    	

        if (bwMeasurementDataList.size()>MAX_NUM_ENTRIES) 
        	bwMeasurementDataList.remove(bwMeasurementDataList.size()-1);
    }//add


    public NetstatBWData getMostRecent(){

        if( bwMeasurementDataList.size()>0 ){
            return (NetstatBWData) bwMeasurementDataList.get(0);
        }else{
            return null;
        }
    }

    public NetstatBWData getBaseBWData() {
    	return baseBWData;
    }
    
    
    public NetstatBWData getValAroundTime(long time) {

    	NetstatBWData bwData = null;
    	if( bwMeasurementDataList.size()>0 ){
    		for (int i = bwMeasurementDataList.size()-1; i >= 0; i--) {
    			if (Math.abs(( (NetstatBWData) bwMeasurementDataList.get(i)).timeRecorded - time )/1000 <= 5) {
    				bwData = (NetstatBWData) bwMeasurementDataList.get(i);
    				break;
    			}
    				
    			//return (NetstatBWData) bwMeasurementDataList.get(0);
    		}
    	}
    	
    	return bwData;
    	
    }
    

    /**
     * Get upto n most recent results.
     * @param n -
     * @return -
     */
    public NetstatBWData[] getMostRecent(int n){

        int size = bwMeasurementDataList.size();
        if( n<size ) size = n;

        if(size==0){
        	NetstatBWData[] def = new NetstatBWData[1];
            def[0] = new NetstatBWData();
            return def;
        }

        NetstatBWData[] retVal = new NetstatBWData[size];
        for(int i=0; i<size; i++){

            retVal[i] = (NetstatBWData) bwMeasurementDataList.get(i);

        }//for
        
        return retVal;
    }

    private ArrayList<NetstatBWData> getDataBetween(long first, long last) {
    	
    	ArrayList<NetstatBWData> retVal = new ArrayList<NetstatBWData>();
    	
    	Iterator<NetstatBWData> iter = bwMeasurementDataList.iterator();
    	while (iter.hasNext()) {
    		NetstatBWData data = iter.next();
    		if ((data.timeRecorded >= first) && (data.timeRecorded <= last))
    			retVal.add(data);
    	}

    	return retVal;
    }
    
    /* returns values in BytesPerSecond */
    public Double getPercentileUploadBetween(long first, long last, float percentile) {
		
    	ArrayList<NetstatBWData> vals = getDataBetween(first, last);
    	
    	double[] v = new double[vals.size()];
    	Double currSent = 0d;
    	Long currTimeSpan = 1l;
    	
    	for (int i=0; i < vals.size()-1; i++) {
			currSent = (double) (vals.get(i).bytesSent - vals.get(i+1).bytesSent);
			currTimeSpan = (vals.get(i).timeRecorded - vals.get(i+1).timeRecorded);
			v[i] = currSent/currTimeSpan;
    	}
    	
    	Percentile pt = new Percentile();
    	pt.setPercentile(percentile);
    	return pt.evaluate(v)*1000;
    	
    }
    
    /* returns values in BytesPerSecond */
    public Double getPercentileDownloadBetween(long first, long last, float percentile) {
		
    	ArrayList<NetstatBWData> vals = getDataBetween(first, last);
    	
    	double[] v = new double[vals.size()];
    	Double currReceived = 0d;
    	Long currTimeSpan = 1l;
    	
    	for (int i=0; i < vals.size()-1; i++) {
			currReceived = (double) (vals.get(i).bytesReceived - vals.get(i+1).bytesReceived);
			currTimeSpan = (vals.get(i).timeRecorded - vals.get(i+1).timeRecorded);
			v[i] = currReceived/currTimeSpan;
    	}
    	
    	Percentile pt = new Percentile();
    	pt.setPercentile(percentile);
    	return pt.evaluate(v)*1000;
    	
    }
    
    /* returns values in BytesPerSecond */
    public Double getFastestUploadBetween(long first, long last) {
    	
    	ArrayList<NetstatBWData> vals = getDataBetween(first, last);
    	Long timeSpan = 1l;
    	Double currSent = 0d;
    	Long currTimeSpan = 1l;
    	Double ret = new Double(-1);
    	
    	for (int i=0; i < vals.size()-1; i++) {

			currSent = (double) (vals.get(i).bytesSent - vals.get(i+1).bytesSent);
			currTimeSpan = (vals.get(i).timeRecorded - vals.get(i+1).timeRecorded);

			if (currTimeSpan != 0 && timeSpan != 0) {
				if (currSent/currTimeSpan > ret)
					ret = currSent/currTimeSpan;
			}
    	}
    	
    	return ret*1000;
    }
    
    /* returns values in BytesPerSecond */
  
    public Double getFastestDownloadBetween(long first, long last) {
    	    	
    	ArrayList<NetstatBWData> vals = getDataBetween(first, last);
    	Long timeSpan = 1l;
    	Double currReceived = 0d;
    	Long currTimeSpan = 1l;
    	Double ret = new Double(-1);
    	
    	for (int i=0; i < vals.size()-1; i++) {

    		currReceived = (double) (vals.get(i).bytesReceived - vals.get(i+1).bytesReceived);
			currTimeSpan = (vals.get(i).timeRecorded - vals.get(i+1).timeRecorded);
			
			if (currTimeSpan != 0 && timeSpan != 0) {
				if (currReceived/currTimeSpan > ret)
					ret = currReceived/currTimeSpan;
			}
    	}
    	
    	return ret*1000;
    }

    @Override
    public String toString() {
    	
    	String ret = new String();
    	Iterator<NetstatBWData> iter = bwMeasurementDataList.iterator();
    	while (iter.hasNext()) {
    		NetstatBWData data = iter.next();
    		ret = ret.concat(data.toString());
    	}
    	
    	return ret;
    }
    
    
    /**
     * data-type.
     */
    public static class NetstatBWData{

    	public long timeRecorded=0;
        public String msg = "";

        public long bytesSent=-1;
        public long bytesReceived=-1;
        
        public long deltaTime=0;
        public long deltaBytesSent=0;
        public long deltaBytesReceived=0;

        //Note: This is used for debugging only.
        public boolean isDebugMode = false;
        public String[] debugLines = null;
        
        public NetstatBWData() {
        	timeRecorded = Util.currentGMTTime();
        }
        
        public NetstatBWData(long sent, long received){
        	timeRecorded = Util.currentGMTTime();
        	bytesSent = sent;
        	bytesReceived = received;
        }
        
        public NetstatBWData(long timeRecorded, long sent, long received){
        	this.timeRecorded = timeRecorded;
        	this.bytesSent = sent;
        	this.bytesReceived = received;
        }
 

        public NetstatBWData(String _msg){
            msg = _msg;
        }

        /**
         * Calculate percentages and deltas based on the raw data.
         * @param prev -
         */
        public void calculate(NetstatBWData prev){

            deltaTime = timeRecorded - prev.timeRecorded;
            deltaBytesSent = bytesSent - prev.bytesSent;
            deltaBytesReceived = bytesReceived - prev.bytesReceived;

        }
        
        @Override
        public String toString() {
        	
        	return "timeRecorded: " + timeRecorded + " bytesSent: " + bytesSent +
        		" bytesReceived: " + bytesReceived + "\n";
        }
        
    }//static class NetstatBWData


}//class
